#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <libusb-1.0/libusb.h>

int handle_dev(libusb_device *dev, uint8_t ep_out, uint8_t ep_in) {
	libusb_device_handle *handle;
	uint8_t data_in[16*1024 + 1];
	int size_in;
	uint8_t data_out[] = {0x55, 0x53, 0x42, 0x43, 0x01, 0x00, 0x00, 0x00, 0x24, 0x00, 0x00, 0x00, 0x80, 0x00, 0x06, 0x12, 0x00, 0x00, 0x00, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	int r = 0;

	libusb_open(dev, &handle);

	printf("Send data\n");
	r = libusb_bulk_transfer(handle,
	      	ep_out, data_out, sizeof(data_out),
	        NULL, 3*1000);

	if (r < 0)
	        goto exit;

	size_in = sizeof(data_in);
	printf("Get data\n");
	r = libusb_bulk_transfer(handle,
	        ep_in, data_in, size_in,
	        &size_in, 3*1000);

	if (r < 0)
	        goto exit;

	size_in = sizeof(data_in);
	printf("Get data\n");
	r = libusb_bulk_transfer(handle,
		ep_in, data_in, size_in,
		&size_in, 3*1000);

	if (r < 0)
		goto exit;

exit:
	libusb_close(handle);

	return r;
}

static libusb_device* get_device(libusb_device **devs, int idVendor, int idProduct)
{
	libusb_device *dev;
	int i = 0, j = 0;
	uint8_t path[8];

	while ((dev = devs[i++]) != NULL) {
		struct libusb_device_descriptor desc;
		int r = libusb_get_device_descriptor(dev, &desc);
		if (r < 0) {
			fprintf(stderr, "failed to get device descriptor");
			return NULL;
		}

		if (desc.idVendor == idVendor && desc.idProduct == idProduct)
			return dev;
	}

	return NULL;
}

int main(int argc, char* argv[]) {
	libusb_device **devs;
	libusb_device *dev;
	ssize_t cnt;
	uint8_t ep_out = 0x02;
	uint8_t ep_in = 0x81;

	int r = 0;

	if (argc < 3) {
		printf("idVend or idProd doesn't specified\n");
		r = 1;
		goto exit;
	}

	if (argc == 5) {
		ep_out = strtol(argv[3], NULL, 16);
		ep_in = strtol(argv[4], NULL, 16);
	}
	
	printf("Libusb init\n");
	r = libusb_init(NULL);
	if (r < 0)
		goto exit;

	printf("Get device list\n");
	cnt = r = libusb_get_device_list(NULL, &devs);
	printf("Number of devices: %ld\n", cnt);
	if (cnt < 0) {
		goto free_libusb;
	}

	dev = get_device(devs, strtol(argv[1], NULL, 16), strtol(argv[2], NULL, 16));
	if (dev == NULL) {
		r = 1;
		goto free_libusb;
	}
	
	r = handle_dev(dev, ep_out, ep_in);


free_libusb:
	printf("Libusb exit\n");
	libusb_exit(NULL);

exit:
	if (r != 0)
		printf("Error\n");
	else
		printf("Success\n");

	return r;
}
